"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const DataLoader = require('dataloader');
class LruCacheWrapper {
    constructor() {
        const lru = require('lru-cache');
        this.cache = lru({ max: 2 });
    }
    clear() { this.cache.reset(); }
    get(key) { return this.cache.get(key); }
    set(key, value) { return this.cache.set(key, value); }
    delete(key) {
        if (this.cache.has(key)) {
            this.cache.del(key);
            return true;
        }
        return false;
    }
}
exports.LruCacheWrapper = LruCacheWrapper;
class MongoEntity {
    constructor(connector, collectionName, cache) {
        this.connector = connector;
        this._collectionName = collectionName;
        this._cache = cache ? cache : MongoEntity.DefaultCache;
    }
    get collection() {
        if (!this._collection) {
            this._collection = this.connector.collection(this._collectionName);
        }
        return this._collection;
    }
    clearUpdateCaches(selector) {
        if (this._updateLoaders) {
            this._updateLoaders.forEach(u => {
                const key = u.selectorKeyFn(selector);
                if (key) {
                    u.dataLoader.clear(selector._id);
                }
                else {
                    u.dataLoader.clearAll();
                }
            });
        }
    }
    clearInsertCaches(selector) {
        if (this._insertLoaders) {
            this._insertLoaders.forEach(i => i.clearAll());
        }
    }
    assignFilter(object, selector, result, include) {
        return (k) => {
            if (selector[k] != include) {
                throw new Error('You cannot combine include and exclude!');
            }
            if (include) {
                if (object[k] !== undefined) {
                    result[k] = object[k];
                }
            }
            else {
                delete (result[k]);
            }
        };
    }
    filter(object, selector) {
        let keys = Object.keys(selector);
        if (keys.length == 0) {
            throw new Error('You need to specify the selector!');
        }
        let include = selector[keys[0]];
        let result = include ? {} : Object.assign({}, object);
        let selectorFunction = this.assignFilter(object, selector, result, include);
        keys.forEach(selectorFunction);
        return result;
    }
    find(selector, fields, skip, limit, timeout) {
        return this.collection.find(selector, fields, skip, limit, timeout);
    }
    findOne(selector, options) {
        return this.collection.findOne(selector, options);
    }
    findOneCached(loader, key, selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (selector) {
                const result = yield loader.load(key);
                return this.filter(result, selector);
            }
            else {
                return loader.load(key);
            }
        });
    }
    findOneCachedById(id, selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._singleLoader) {
                this._singleLoader = this.createLoader((loadId) => {
                    return this.collection.findOne({ _id: loadId });
                }, this.addCacheToOptions({ clearOnUpdate: true, selectorKeyFn: (a) => a._id }));
            }
            return this.findOneCached(this._singleLoader, id, selector);
        });
    }
    findManyCached(loader, key, selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (selector) {
                const result = yield loader.load(key);
                return result.map(r => this.filter(r, selector));
            }
            else {
                return loader.load(key);
            }
        });
    }
    findAllCached(selector) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._multiLoader) {
                this._multiLoader = this.createLoader(() => this.collection.find().toArray(), this.addCacheToOptions({ clearOnInsert: true }));
            }
            return this.findManyCached(this._multiLoader, 'ALL', selector);
        });
    }
    addCacheToOptions(options) {
        if (options.cacheMap) {
            return options;
        }
        if (this._cache) {
            options.cacheMap = this._cache;
        }
        return options;
    }
    insertOne(document) {
        this.clearInsertCaches(document);
        return this.collection.insertOne(document);
    }
    insertMany(document) {
        this.clearInsertCaches(document);
        return this.collection.insertMany(document);
    }
    deleteOne(selector, many = false) {
        this.clearInsertCaches(selector);
        return this.collection.deleteOne(selector);
    }
    deleteMany(selector, many = false) {
        this.clearInsertCaches(selector);
        return this.collection.deleteMany(selector);
    }
    dispose() {
        return this.deleteMany({}, true);
    }
    updateOne(selector, update, options) {
        this.clearUpdateCaches(selector);
        return this.collection.updateOne(selector, update, options);
    }
    updateMany(selector, update, options) {
        this.clearUpdateCaches(selector);
        return this.collection.updateMany(selector, update, options);
    }
    createLoader(selectorFunction, options) {
        const opts = options;
        if (opts.cacheMap == 'lru') {
            opts.cacheMap = new LruCacheWrapper();
        }
        const loader = new DataLoader((keys) => {
            return Promise.all(keys.map(selectorFunction));
        }, opts);
        if (options) {
            if (options.clearOnInsert) {
                if (!this._insertLoaders) {
                    this._insertLoaders = [];
                }
                this._insertLoaders.push(loader);
            }
            if (options.clearOnUpdate) {
                if (!this._updateLoaders) {
                    this._updateLoaders = [];
                }
                if (!options.selectorKeyFn) {
                    throw new Error('You need to provide cache key function to determine when cache needs to be updated');
                }
                this._updateLoaders.push({
                    dataLoader: loader,
                    selectorKeyFn: options.selectorKeyFn
                });
            }
        }
        return loader;
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = MongoEntity;
