"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const assert = require("power-assert");
const sinon = require("sinon");
const proxyquire = require("proxyquire");
const testing_1 = require("../testing");
const mongo_entity_1 = require("../mongo_entity");
const host = process.env.MONGODB_HOST || '127.0.0.1';
const port = process.env.MONGODB_PORT || 27017;
describe('entity', () => {
    it('contains connector and collection name', () => __awaiter(this, void 0, void 0, function* () {
        var connector = yield testing_1.getConnector();
        var collectionName = 'name';
        var entity = new mongo_entity_1.default(connector, collectionName);
        assert.equal(entity.collection.collectionName, collectionName);
        assert.equal(entity.connector, connector);
        assert.notEqual(entity.collection, null);
    }));
    it('is possible to initialise entity with custom cache', function () {
        const cache = { local: '' };
        const entity = new mongo_entity_1.default(null, 'name', cache);
        assert.equal(entity['_cache'], cache);
        const globalCache = { global: '1' };
        mongo_entity_1.default.DefaultCache = globalCache;
        const otherEntity = new mongo_entity_1.default(null, 'name');
        assert.equal(otherEntity['_cache'], globalCache);
        mongo_entity_1.default.DefaultCache = null;
    });
    describe('find', () => {
        it('can find a multiple record', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => {
                const find = sinon.spy(entity.collection, 'find');
                const selector = {};
                const fields = {};
                const skip = 1;
                const limit = 2;
                const timeout = 1000;
                entity.find(selector, fields, skip, limit, timeout);
                assert(find.calledWithExactly(selector, fields, skip, limit, timeout));
            });
        }));
    });
    describe('findOne', () => {
        it('can find a single record', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => {
                const find = sinon.spy(entity.collection, 'find');
                const selector = {};
                const options = {};
                entity.findOne(selector, options);
                assert(find.calledWithExactly(selector, options));
            });
        }));
    });
    describe('findOneCachedById', () => {
        it('can find and cache results of finding a single item', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                const find = sinon.spy(entity.collection, 'findOne');
                let result = yield entity.findOne({ _id: '1' });
                result = yield entity.findOne({ _id: '1' });
                assert.deepEqual(result, { _id: '1' });
                assert(find.calledTwice);
                find.reset();
                result = yield entity.findOneCachedById('1');
                result = yield entity.findOneCachedById('1');
                assert(find.calledOnce);
            }), {
                entities: [{
                        data: [{ _id: '1' }]
                    }]
            });
        }));
        it('clears the related cache when a document is updated', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    const findSpy = sinon.spy(entity.collection, 'findOne');
                    const cacheSpy = sinon.spy(entity, 'clearUpdateCaches');
                    yield entity.insertOne({ _id: '1', file: 'foo' });
                    yield entity.insertOne({ _id: '2', file: 'bar' });
                    let foo = yield entity.findOneCachedById('1');
                    let bar = yield entity.findOneCachedById('2');
                    const selector = { _id: '1' };
                    entity.updateOne(selector, { $set: { file: 'boo' } });
                    assert(cacheSpy.calledWith(selector));
                    findSpy.reset();
                    foo = yield entity.findOneCachedById('1');
                    assert.deepEqual(foo, { _id: '1', file: 'boo' });
                    assert(findSpy.calledOnce);
                    findSpy.reset();
                    bar = yield entity.findOneCachedById('2');
                    assert(findSpy.notCalled);
                }));
            });
        });
        it('clears all caches when a document is updated and selector is unknown', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    let foo = yield entity.findOneCachedById('1');
                    let bar = yield entity.findOneCachedById('2');
                    entity.updateOne({ file: 'foo' }, { $set: { file: 'boo' } });
                    const findSpy = sinon.spy(entity.collection, 'findOne');
                    foo = yield entity.findOneCachedById('1');
                    bar = yield entity.findOneCachedById('2');
                    assert(findSpy.calledTwice);
                }));
            });
        });
        it('findOneCached can filter returned results', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                yield entity.insertOne({ _id: '00', a: '1', b: '2', c: '3' });
                let result = yield entity.findOneCachedById('00', { b: 1 });
                assert.deepEqual(result, { b: '2' });
            }));
        }));
    });
    describe('addCacheToOptions', () => {
        it('adds entitys cache to options', function () {
            const cache = { g: 1 };
            const entity = new mongo_entity_1.default(null, null, cache);
            const options = {};
            const res1 = entity.addCacheToOptions(options);
            assert.deepEqual(res1, { cacheMap: cache });
            const optionsWithCache = { cacheMap: {} };
            const res2 = entity.addCacheToOptions(optionsWithCache);
            assert.deepEqual(res2, optionsWithCache);
        });
    });
    describe('filter', () => {
        it('can filter objects properties', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = { a: 1, b: 2, c: 3 };
            const resultWith = entity.filter(obj, { a: 1, c: 1 });
            assert.deepEqual(resultWith, { a: 1, c: 3 });
            const resultWithout = entity.filter(obj, { a: 0, c: 0 });
            assert.deepEqual(resultWithout, { b: 2 });
        });
        it('does not include undefined elements', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = { a: 1, b: 2, c: 3 };
            const resultWith = entity.filter(obj, { a: 1, c: 1, d: 1 });
            assert.deepEqual(resultWith, { a: 1, c: 3 });
        });
        it('throws error when no filter is specified', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = {};
            assert.throws(() => entity.filter(obj, {}), /You need to specify the selector!/);
        });
        it('throws error when you mix include exclude', function () {
            const entity = new mongo_entity_1.default(null, null);
            const obj = {};
            assert.throws(() => entity.filter(obj, { a: 1, b: 0 }), /You cannot combine include and exclude!/);
        });
    });
    describe('findManyCached', () => {
        it('can find and cache results of finding multiple items', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                entity.collection.insertOne({ _id: '1' });
                const find = sinon.spy(entity.collection, 'find');
                let result = yield entity.findAllCached();
                result = yield entity.findAllCached();
                assert.deepEqual(result, [{ _id: '1' }]);
                assert(find.calledOnce);
            }));
        }));
        it('findManyCached can filter returned results', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                yield entity.insertMany([
                    { _id: '00', a: '1', b: '2', c: '3' },
                    { _id: '01', a: '1', b: '2', c: '3' }
                ]);
                let result = yield entity.findAllCached({ a: 1, c: 1 });
                assert.deepEqual(result, [{ a: '1', c: '3' }, { a: '1', c: '3' }]);
            }));
        }));
        it('clears the cache when a new document is inserted', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    const find = sinon.spy(entity.collection, 'find');
                    entity.insertOne({ _id: '1' });
                    let result = yield entity.findAllCached();
                    entity.insertOne({ _id: '2' });
                    result = yield entity.findAllCached();
                    assert(find.calledTwice);
                    assert.deepEqual(result, [{ _id: '1' }, { _id: '2' }]);
                }));
            });
        });
    });
    describe('delete', () => {
        it('can delete a single document', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                yield entity.insertOne({ _id: '00', a: '1', b: '2', c: '3' });
                yield entity.insertOne({ _id: '01', a: '1', b: '2', c: '3' });
                let result = yield entity.findAllCached();
                assert.equal(result.length, 2);
                yield entity.deleteOne({ _id: '00' });
                result = yield entity.findAllCached();
                assert.equal(result.length, 1);
            }));
        }));
        it('can delete multiple documents', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    yield entity.insertOne({ _id: '00', a: '1', b: '2', c: '3' });
                    yield entity.insertOne({ _id: '01', a: '1', b: '2', c: '3' });
                    yield entity.deleteMany();
                    const result = yield entity.findAllCached();
                    assert.equal(result.length, 0);
                }));
            });
        });
    });
    describe('dispose', () => {
        it('deletes all records', sinon.test(function () {
            const entity = new mongo_entity_1.default(null, 'name');
            const stub = this.stub(entity, 'deleteMany');
            entity.dispose();
            sinon.assert.calledWith(stub, {}, true);
        }));
    });
    describe('testing', () => {
        it('can execute test with multiple entities', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((e1, e2, e3) => {
                    assert(e1);
                    assert.equal(e1.collection.collectionName, 'e1');
                    assert(e2);
                    assert.equal(e2.collection.collectionName, 'e2');
                    assert(e3);
                    assert.equal(e3.collection.collectionName, 'e3');
                }, { entities: [{ name: 'e1' }, { name: 'e2' }, { name: 'e3' }] });
            });
        });
    });
    describe('custom loaders', () => {
        it('throws error when selectorKeyFunction is not specified with updates', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => {
                assert.throws(() => entity.createLoader(() => { }, { clearOnUpdate: true }), /You need to provide cache key function to determine when cache needs to be updated/);
            });
        }));
        it('can create a new data-loader with a custom cache', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    const records = [{ _id: 1, name: 'A' }, { _id: 2, name: 'B' }, { _id: 3, name: 'C' }];
                    entity.insertMany(records);
                    const loader = entity.createLoader((name) => {
                        return entity.collection.findOne({ name });
                    }, {
                        cacheMap: 'lru',
                        clearOnInsert: true,
                        clearOnUpdate: true,
                        selectorKeyFn: (a) => a.name
                    });
                    const spy = sinon.spy(entity.collection, 'findOne');
                    yield entity.findOneCached(loader, 'A');
                    yield entity.findOneCached(loader, 'A');
                    yield entity.findOneCached(loader, 'B');
                    yield entity.findOneCached(loader, 'B');
                    yield entity.findOneCached(loader, 'A');
                    sinon.assert.calledTwice(spy);
                    spy.reset();
                    yield entity.updateOne({ _id: 1 }, { $set: { name: 'D' } });
                    const a = yield entity.findOneCached(loader, 'A');
                    assert.equal(a, null);
                    sinon.assert.calledOnce(spy);
                    spy.reset();
                    yield entity.findOneCached(loader, 'C');
                    yield entity.findOneCached(loader, 'A');
                    sinon.assert.calledOnce(spy);
                    spy.reset();
                    yield entity.findOneCached(loader, 'B');
                    sinon.assert.calledOnce(spy);
                    spy.reset();
                    entity.insertOne({ _id: 4, name: 'D' });
                    yield entity.findOneCached(loader, 'B');
                    sinon.assert.calledOnce(spy);
                }));
            });
        });
    });
    describe('LruCacheWrapper', () => {
        it('maps lru-cache functions', sinon.test(function () {
            function cacheStub() {
                return {
                    get: sinon.stub(),
                    set: sinon.stub(),
                    reset: sinon.stub(),
                    has: sinon.stub(),
                    del: sinon.stub()
                };
            }
            const { LruCacheWrapper } = proxyquire('../mongo_entity', { 'lru-cache': cacheStub });
            const cacheWrapper = new LruCacheWrapper();
            cacheWrapper.clear();
            sinon.assert.calledOnce(cacheWrapper.cache.reset);
            cacheWrapper.get();
            sinon.assert.calledOnce(cacheWrapper.cache.get);
            cacheWrapper.set();
            sinon.assert.calledOnce(cacheWrapper.cache.set);
            cacheWrapper.cache.has.returns(true);
            const positiveResult = cacheWrapper.delete('1');
            assert(positiveResult);
            sinon.assert.calledWith(cacheWrapper.cache.del, '1');
            cacheWrapper.cache.has.returns(false);
            const negativeResult = cacheWrapper.delete('1');
            assert.equal(negativeResult, false);
        }));
    });
});
