"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const sinon = require("sinon");
const mongo_entity_1 = require("../mongo_entity");
const testing_1 = require("../testing");
const assert = require("power-assert");
const proxyquire = require("proxyquire");
describe('Testing Helpers', () => {
    describe('withContext', () => {
        it('initializes context with connection, executes tests and cleans up afterwards', function () {
            return __awaiter(this, void 0, void 0, function* () {
                function initContext(conn) {
                    return {
                        entity: new mongo_entity_1.default(conn, 'col'),
                        other: {}
                    };
                }
                let inContext = null;
                let deleteSpy = null;
                yield testing_1.withContext((context) => __awaiter(this, void 0, void 0, function* () {
                    assert(context.entity);
                    deleteSpy = sinon.spy(context.entity, 'deleteMany');
                    inContext = context;
                }), initContext);
                sinon
                    .assert
                    .calledOnce(deleteSpy);
            });
        });
        it('throws exception but also cleans up', function () {
            return __awaiter(this, void 0, void 0, function* () {
                function initContext(conn) {
                    return {
                        entity: new mongo_entity_1.default(conn, 'col')
                    };
                }
                let inContext = null;
                let deleteSpy = null;
                try {
                    yield testing_1.withContext((context) => __awaiter(this, void 0, void 0, function* () {
                        deleteSpy = sinon.spy(context.entity, 'deleteMany');
                        inContext = context;
                        throw new Error('Error');
                    }), initContext);
                }
                catch (ex) {
                    assert.equal(ex.message, 'Error');
                }
                sinon
                    .assert
                    .calledOnce(deleteSpy);
            });
        });
        it('can run from the globally declared contextFunction', () => __awaiter(this, void 0, void 0, function* () {
            const initContext = sinon
                .stub()
                .returns({});
            testing_1.config({ initContext });
            yield testing_1.withContext(() => { });
            sinon
                .assert
                .calledOnce(initContext);
        }));
        it('can run in the disconnected state', () => __awaiter(this, void 0, void 0, function* () {
            const MongoStub = {
                MongoClient: sinon.stub()
            };
            const { withContext, config } = proxyquire('../testing', { 'mongodb': MongoStub });
            const initContext = sinon.stub().returns({});
            config({ initContext });
            yield withContext((context) => { }, null, true);
            sinon.assert.notCalled(MongoStub.MongoClient);
        }));
    });
    describe('Mocha helpers', () => {
        describe('itWithContext', () => {
            testing_1.itWithContext('runs', (context) => {
                assert(true);
            });
        });
        describe('itWithEntity', () => {
            testing_1.itWithEntity('runs', (context) => {
                assert(true);
            });
        });
    });
    describe('disposeDb', () => {
        it('closes connection', sinon.test(function () {
            return __awaiter(this, void 0, void 0, function* () {
                const dbSpy = {
                    dropDatabase: this.spy(),
                    close: this.spy()
                };
                const MongoStub = {
                    MongoClient: {
                        connect: this.stub().returns(dbSpy)
                    }
                };
                const { getDb, disposeDb } = proxyquire('../testing', { 'mongodb': MongoStub });
                yield getDb();
                yield disposeDb(true);
                sinon.assert.calledOnce(dbSpy.dropDatabase);
                sinon.assert.calledOnce(dbSpy.close);
            });
        }));
    });
});
